!=====================================================================================
!=====================================================================================
Subroutine pillar()

use variables

implicit none

real*8 beta, dstar, cond1RHS, cond2RHS, A0, A1, A2, A, B1, B2, B, angleGamma, angleAlpha, anglePhi, Rstar_eff

! calculate the dimensionless sagging radius
RsagStar(lvl) = (hStar(lvl)**2 + 0.25d0*(dble(j(lvl))-dble(i(lvl)))**2)/(2.d0*hStar(lvl))

! calculate the beta angle
!beta = asin((dble(j(lvl))-dble(i(lvl)))/(2.d0*RsagStar(lvl)))

! calculate the dimensionless distance d
dstar = (dble(j(lvl)) - dble(i(lvl)))/2.d0
if(dstar.lt.1d-300)then
   write(*,*)"pillar width becomes larger than distance between pillars"
   stop
endif

cond1RHS = dstar/sin(thetaY)


! ----- first condition: whether the contact point is A1 or N1 -----

   ! ----- second condition: Given that we have A1, check whether the liquid touches the surface or not -----
   if(hStar(lvl).lt.HeightStar(lvl))then ! NO TOUCHING OF THE SURFACE

      case = 1

      fsl(lvl) = dble(i(lvl))/dble(j(lvl))
      beta = asin((dble(j(lvl))-dble(i(lvl)))/(2.d0*RsagStar(lvl)))
      fla(lvl) = (RsagStar(lvl)*2.d0*beta)/dble(j(lvl))
!      beta = (pi - thetaY)
!      fla(lvl) = (cond1RHS*2.d0*beta)/dble(j(lvl))

   else  ! given that we have A1, the liquid TOUCHES the surface

      case = 3

!      angleAlpha = acos((1.d0 + sqrt(RsagStar(lvl)**2 - dstar**2))/RsagStar(lvl))
      angleAlpha = acos((HeightStar(lvl) + sqrt(RsagStar(lvl)**2 - dstar**2))/RsagStar(lvl))
!      angleAlpha = acos((1.d0 + sqrt(cond1RHS**2 -dstar**2))/RsagStar(lvl))

      fsl(lvl) = dble(i(lvl))/dble(j(lvl)) + 2.d0*RsagStar(lvl)*(sqrt(1.d0-(cos(angleAlpha))**2))/dble(j(lvl))
!      beta = (pi - thetaY)
      beta = asin((dble(j(lvl))-dble(i(lvl)))/(2.d0*RsagStar(lvl)))
      fla(lvl) = 2.d0*RsagStar(lvl)*(beta-angleAlpha)/dble(j(lvl))

   endif


fsl_org(lvl) = dble(i(lvl))/dble(j(lvl))
fla_org(lvl) = 1.d0 - (dble(i(lvl))/dble(j(lvl)))


return
end
!=====================================================================================
!=====================================================================================



!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Subroutine fiber()

use variables

implicit none

real*8 dstar, cond1RHS, cond1LHS, cond2RHS, A0, A1, A2, A, B1, B2, B, angleGamma, angleAlpha, Rstar_eff



! calculate the dimensionless distance d
dstar = (dble(j(lvl))/2.d0) - dble(i(lvl))*sin(thetaY)
if(dstar.lt.1d-300)then
   write(*,*)"fiber radius becomes larger than fiber distance"
   stop
endif


! calculate the dimensionless sagging radius
RsagStar(lvl) = (hStar(lvl)**2 + dstar**2)/(2.d0*hStar(lvl))

! ---------------------- Start checking conditions ---------------------
! ----first condition: whether the liquid touches the base or not ------
cond1LHS = dble(i(lvl))*(1.d0-cos(thetaY))
cond1RHS = RsagStar(lvl) - sqrt((RsagStar(lvl)**2) - (dstar**2))

cond2RHS = dstar/sin(thetaY)

if(cond1LHS.gt.cond1RHS)then ! then the liquid DOES NOT touch the base of the surface
   
   ! ----- second condition: whether the contact point is A1 or N1 -----
   if(RsagStar(lvl).ge.cond2RHS)then   ! A1

      case = 1   

      fsl(lvl) = 2.d0*dble(i(lvl))*(pi - thetaY)/dble(j(lvl))

      fla(lvl) = 2.d0*RsagStar(lvl)*asin(dstar/RsagStar(lvl))/dble(j(lvl))


   elseif(RsagStar(lvl).lt.cond2RHS)then  ! N1

      Rstar_eff = cond2RHS + (cond2RHS - RsagStar(lvl))  ! Definition of Rstar_effective
      
!      A0 = (sqrt(RsagStar(lvl)**2 - dstar**2) - dble(i(lvl))*cos(thetaY))
      A0 = (sqrt(cond2RHS**2 - dstar**2) - dble(i(lvl))*cos(thetaY))
      A1 = 4.d0*A0
!      A2 = ( ((dble(i(lvl))**2) + (dble(j(lvl))/2.d0)**2 + (A0**2) - (RsagStar(lvl)**2))/dble(i(lvl)) ) + dble(j(lvl))
      A2 = ( ((dble(i(lvl))**2) + (dble(j(lvl))/2.d0)**2 + (A0**2) - (Rstar_eff**2))/dble(i(lvl)) ) + dble(j(lvl))
      A = A1/A2

!      B1 = ( ((dble(i(lvl))**2) + (dble(j(lvl))/2.d0)**2 + (A0**2) - (RsagStar(lvl)**2))/dble(i(lvl)) ) - dble(j(lvl))
      B1 = ( ((dble(i(lvl))**2) + (dble(j(lvl))/2.d0)**2 + (A0**2) - (Rstar_eff**2))/dble(i(lvl)) ) - dble(j(lvl))
!      B2 = ( ((dble(i(lvl))**2) + (dble(j(lvl))/2.d0)**2 + (A0**2) - (RsagStar(lvl)**2))/dble(i(lvl)) ) + dble(j(lvl))
      B2 = ( ((dble(i(lvl))**2) + (dble(j(lvl))/2.d0)**2 + (A0**2) - (Rstar_eff**2))/dble(i(lvl)) ) + dble(j(lvl))
      B = B1/B2

      if(A.gt.1d-300)then
         angleGamma = 2.d0*atan(0.5d0*(A - sqrt(A**2 - 4.d0*B)))
      elseif(A.lt.1d-300)then
!         angleGamma = 2.d0*atan(0.5d0*(A + sqrt(A**2 - 4.d0*B)))
         angleGamma = 2.d0*atan(0.5d0*(A - sqrt(A**2 - 4.d0*B)))
      endif
 
      case = 2

      fsl(lvl) = 2.d0*(dble(i(lvl))*(pi - thetaY)/dble(j(lvl))) + 2.d0*(dble(i(lvl))*(thetaY-(pi/2.d0)-angleGamma)/dble(j(lvl)))

!      fla(lvl) = 2.d0*RsagStar(lvl)*acos((sqrt(RsagStar(lvl)**2 - dstar**2) - dble(i(lvl))*cos(thetaY) - dble(i(lvl))*sin(angleGamma))/RsagStar(lvl))/dble(j(lvl))
      fla(lvl) = 2.d0*Rstar_eff*acos((sqrt(cond2RHS**2 - dstar**2) - dble(i(lvl))*cos(thetaY) - &
                                      dble(i(lvl))*sin(angleGamma))/Rstar_eff)/dble(j(lvl))

   endif

elseif(cond1LHS.lt.cond1RHS)then ! then the liquid TOUCHES the surface

   ! ----- second condition: whether the contact point is A1 or N1 -----
   if(RsagStar(lvl).ge.cond2RHS)then  ! A1

      case = 3

      fsl(lvl) = (2.d0*dble(i(lvl))*(pi - thetaY)/dble(j(lvl))) + &
                 (2.d0*RsagStar(lvl)*sqrt(1.d0-(sqrt(1.d0 - (dstar/RsagStar(lvl))**2) + &
                      (dble(i(lvl))/RsagStar(lvl))*(1.d0-cos(thetaY)))**2)/dble(j(lvl)))

      fla(lvl) = (2.d0*RsagStar(lvl)/dble(j(lvl)))*(asin(dstar/RsagStar(lvl)) - &
                  acos((dble(i(lvl))/RsagStar(lvl))*(1.d0 - cos(thetaY) +  &
                  (RsagStar(lvl)/dble(i(lvl)))*sqrt(1.d0 - (dstar/RsagStar(lvl))**2))))

   elseif(RsagStar(lvl).lt.cond2RHS)then  ! N1

      Rstar_eff = cond2RHS + (cond2RHS - RsagStar(lvl))  ! Definition of Rstar_effective

!      A0 = (sqrt(RsagStar(lvl)**2 - dstar**2) - dble(i(lvl))*cos(thetaY))
      A0 = (sqrt(cond2RHS**2 - dstar**2) - dble(i(lvl))*cos(thetaY))
      A1 = 4.d0*A0
!      A2 = ( ((dble(i(lvl))**2) + (dble(j(lvl))/2.d0)**2 + (A0**2) - (RsagStar(lvl)**2))/dble(i(lvl)) ) + dble(j(lvl))
      A2 = ( ((dble(i(lvl))**2) + (dble(j(lvl))/2.d0)**2 + (A0**2) - (Rstar_eff**2))/dble(i(lvl)) ) + dble(j(lvl))
      A = A1/A2

!      B1 = ( ((dble(i(lvl))**2) + (dble(j(lvl))/2.d0)**2 + (A0**2) - (RsagStar(lvl)**2))/dble(i(lvl)) ) - dble(j(lvl))
      B1 = ( ((dble(i(lvl))**2) + (dble(j(lvl))/2.d0)**2 + (A0**2) - (Rstar_eff**2))/dble(i(lvl)) ) - dble(j(lvl))
!      B2 = ( ((dble(i(lvl))**2) + (dble(j(lvl))/2.d0)**2 + (A0**2) - (RsagStar(lvl)**2))/dble(i(lvl)) ) + dble(j(lvl))      
      B2 = ( ((dble(i(lvl))**2) + (dble(j(lvl))/2.d0)**2 + (A0**2) - (Rstar_eff**2))/dble(i(lvl)) ) + dble(j(lvl))
      B = B1/B2

!      angleAlpha = acos((dble(i(lvl))/RsagStar(lvl)) * (1.d0 - cos(thetaY) +  (RsagStar(lvl)/dble(i(lvl)))*sqrt(1.d0 - (dstar/RsagStar(lvl))**2)))
      angleAlpha = acos((dble(i(lvl))/Rstar_eff) * (1.d0 - cos(thetaY) +  &
                   (RsagStar(lvl)/dble(i(lvl)))*sqrt(1.d0 - (dstar/RsagStar(lvl))**2)))

!      angleGamma = 2.d0*atan(0.5d0*(-A + sqrt(A**2 - 4.d0*B)))
      if(A.gt.1d-300)then
         angleGamma = 2.d0*atan(0.5d0*(-A + sqrt(A**2 - 4.d0*B)))
      elseif(A.lt.1d-300)then
         angleGamma = 2.d0*atan(0.5d0*(-A - sqrt(A**2 - 4.d0*B)))
      endif

      case = 4

!      fsl(lvl) = (2.d0*dble(i(lvl))*(pi-thetaY)/dble(j(lvl))) + (2.d0*RsagStar(lvl)*sin(angleAlpha)/dble(j(lvl))) + &
!                 (2.d0*dble(i(lvl))*(thetaY - (pi/2.d0) + angleGamma)/dble(j(lvl))) 
       fsl(lvl) = (2.d0*dble(i(lvl))*(pi-thetaY)/dble(j(lvl))) + (2.d0*Rstar_eff*sin(angleAlpha)/dble(j(lvl))) + &
                  (2.d0*dble(i(lvl))*(thetaY - (pi/2.d0) + angleGamma)/dble(j(lvl)))


!      fla(lvl) = (2.d0*RsagStar(lvl)/dble(j(lvl))) * &
!                     (asin(((dble(j(lvl))/2.d0) - dble(i(lvl))*cos(angleGamma))/RsagStar(lvl)) - &
!                      acos((dble(i(lvl))/RsagStar(lvl))*(1.d0 - cos(thetaY) +  (RsagStar(lvl)/dble(i(lvl)))*sqrt(1.d0 - (dstar/RsagStar(lvl))**2))))
      fla(lvl) = (2.d0*Rstar_eff/dble(j(lvl))) * &
                     (acos(( sqrt((RsagStar(lvl)**2) - (dstar**2)) - dble(i(lvl))*cos(thetaY) + &
                         dble(i(lvl)*sin(angleGamma)))/Rstar_eff) - acos((dble(i(lvl))/Rstar_eff)* &
                         (1.d0 - cos(thetaY) +  (RsagStar(lvl)/dble(i(lvl)))*sqrt(1.d0 - (dstar/RsagStar(lvl))**2))))



   endif

endif


fsl_org(lvl) = (pi - thetaY)*dble(i(lvl))/dble(j(lvl))
fla_org(lvl) = 1.d0 - 2.d0*(dble(i(lvl))/dble(j(lvl)))*sin(thetaY)


return
end
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++



!######################################################################################################################################
!######################################################################################################################################
Subroutine sinusoidal()

use variables

implicit none

real*8 condLHS, condRHS, x1Star, ksi1, aa, bb, cc, &
       c1, c2, c3, c4, c5, c6, c7, factor1, denominator1, &
       firstterm, secondterm, denominator2, thirdterm, sol1, &
       sol2, sol3, sol4, goodsol1, temp1, temp2, temp3, temp4, &
       epsilon, ksi, nu, par1, par2, par3, junk, ee1, ee2, ee3
character*12 wrt_n1, wrt_n2, wrt_n3, wrt_n4


epsilon = 3.d-2   ! something small (threshold value)

condLHS = abs(tan(pi-thetaY))
condRHS = (2.d0*pi*dble(i(lvl))/dble(j(lvl)))

! -------------- check condition: whether we have full or partial wetting -----------------
if(condLHS.lt.condRHS)then ! partial wetting

   case = 1
   ! calculate the dimensionless x1Star
   x1Star = (dble(j(lvl))/(2.d0*pi))*asin(tan(pi-thetaY)*dble(j(lvl))/(2.d0*pi*dble(i(lvl))))
   ! consider only possitive ksis (take absolute value). Solution is symmetric about 0
   ksi1 = abs(asin(tan(pi-thetaY)*dble(j(lvl))/(2.d0*pi*dble(i(lvl)))))

   ! calculate the dimensionless sagging radius
   RsagStar(lvl) = (hStar(lvl)**2 + x1Star**2)/(2.d0*hStar(lvl))

   ! --------------- CHECK WHETHER WE HAVE ONE ROOT OR TWO ----------------
   ! ------------------------CALCULATE PARAMETERS--------------------------

   ! ----------------------------- 1st pass -------------------
   aa = (RsagStar(lvl)/dble(i(lvl))); bb = (dble(j(lvl))/(2.d0*pi*RsagStar(lvl)))**2;
   cc = cos(ksi1)-(RsagStar(lvl)/dble(i(lvl)))*sqrt(1.d0-(dble(j(lvl))*ksi1/(2.d0*pi*RsagStar(lvl)))**2)
   ! ----------------------------------------------------------

   ! ----------------------------- 2nd pass -------------------
   c1 = (-4.d0 + cc); c2 = (1.d0 - 3.d0*(aa**2) + 18*(aa**2)*bb + 4.d0*cc + 4.d0*(cc**2))
   c3 = c1**3; c4 = (-1.d0 + (aa**2)*bb + cc); c5 = c4**2; c6 = (-1.d0 + (aa**2) + 2.d0*cc - (cc**2))
   c7 = (2304.d0 - 6912.d0*(aa**2) + 41472.d0*(aa**2)*bb + 9216.d0*cc + 9216.d0*(cc**2))**3
   ! ----------------------------------------------------------

   ! ----------------------------- 3rd pass -------------------
   factor1 = -221184.d0*c3 - 5971968.d0*c1*c4 + 8957952.d0*c5 - 8957952.d0*c6 - 1990656.d0*c1*c6
   denominator1 = (factor1 + sqrt(-4.d0*c7 + (factor1**2)))**(1.d0/3.d0)

   firstterm = 144.d0 + 32.d0*c1 + (967.6193663193d0*c2/denominator1) + 0.264566842d0*denominator1
   secondterm = 288.d0 + 64.d0*c1 - (967.6193663193d0*c2/denominator1) - 0.264566842d0*denominator1

   denominator2 = 4.d0*sqrt(firstterm)
   thirdterm = (13824.d0 + 4608.d0*c1 - 4608.d0*c4)/denominator2
   ! write(333,20) factor1, denominator1, firstterm, secondterm, denominator2, thirdterm
   ! ----------------------------------------------------------

   ! ----------------------------------------------------------------------
   ! ----------------------------------------------------------------------

   sol1 = sqrt(6.d0 - 0.5d0*sqrt(firstterm) - 0.5d0*sqrt(secondterm - thirdterm))
   sol2 = sqrt(6.d0 - 0.5d0*sqrt(firstterm) + 0.5d0*sqrt(secondterm - thirdterm))
   sol3 = sqrt(6.d0 + 0.5d0*sqrt(firstterm) - 0.5d0*sqrt(secondterm + thirdterm))
   sol4 = sqrt(6.d0 + 0.5d0*sqrt(firstterm) + 0.5d0*sqrt(secondterm + thirdterm))

   write(wrt_n1,'(G12.5)') sol1
   write(wrt_n2,'(G12.5)') sol2
   write(wrt_n3,'(G12.5)') sol3
   write(wrt_n4,'(G12.5)') sol4


   goodsol1 = ksi1
   ! intialize temp solutions with unnaturally high values
   temp1 = 1000.d0; temp2 = 1000.d0; temp3 = 1000.d0; temp4 = 1000.d0;
   ! check sol1
   if(Trim(AdjustL(wrt_n1)).ne.'NaN')then         ! we have a meaningful solution
      if(abs(sol1-goodsol1).gt.epsilon)then ! we found the second solution besides ksi1
         temp1 = min(abs(sol1), goodsol1)
         go to 35
      endif
   endif

   if(Trim(AdjustL(wrt_n2)).ne.'NaN')then         ! we have a meaningful solution
      if(abs(sol2-goodsol1).gt.epsilon)then ! we found the second solution besides ksi1
         temp2 = min(abs(sol2), goodsol1)
         go to 35
      endif
   endif

   if(Trim(AdjustL(wrt_n3)).ne.'NaN')then         ! we have a meaningful solution
      if(abs(sol3-goodsol1).gt.epsilon)then ! we found the second solution besides ksi1
         temp3 = min(abs(sol3), goodsol1)
         go to 35
      endif
   endif

   if(Trim(AdjustL(wrt_n4)).ne.'NaN')then         ! we have a meaningful solution
      if(abs(sol4-goodsol1).gt.epsilon)then ! we found the second solution besides ksi1
         temp4 = min(abs(sol4), goodsol1)
         go to 35
      endif
   endif

35  continue
   ksi = min(temp1, temp2, temp3, temp4, ksi1)

   ! calculate parameter nu, par1, par2, par3 that go into the elliptic integrals
   nu = (2.d0*pi*dble(i(lvl))/dble(j(lvl)))
   par1 = (nu/sqrt(1.d0+(nu**2)))
   par2 = (ksi - (pi/2.d0))
   par3 = ((pi/2.d0) - ksi)

   ! call subroutines to calculate (in)complete elliptic integral of the second kind
   ! par2 and par3 need to be in degrees
   par2 = par2*(180.d0/pi); par3 = par3*(180.d0/pi);
   call elit(par1,par2,junk,ee1)          ! incomplete elliptic integral of the second kind (phi = ξ - π/2)
   call elit(par1,par3,junk,ee2)          ! incomplete elliptic integral of the second kind (phi = π/2 - ξ)
   call comelp(par1,junk,ee3)             ! complete elliptic integral of the second kind

   ! ------------------- check wheter xi is gt or lt pi/2 ---------------------
   if(ksi.gt.(pi/2.d0))then

      fsl(lvl) = abs((sqrt(1.d0+(nu**2))/pi)*(ee3 - ee1))
      fsl_org(lvl) = abs((sqrt(1.d0+(nu**2))/pi)*(ee3 - ee1))

   elseif(ksi.lt.(pi/2.d0))then

      fsl(lvl) = abs((sqrt(1.d0+(nu**2))/pi)*(ee3 - ee2))
      fsl_org(lvl) = abs((sqrt(1.d0+(nu**2))/pi)*(ee3 + ee1))

   endif
   ! --------------------------------------------------------------------------

   fla(lvl) = (2.d0*RsagStar(lvl)/dble(j(lvl)))*asin(dble(j(lvl))*ksi/(2.d0*pi*RsagStar(lvl)))
   !fla_orig = abs(asin(tan(pi-thetaY)*dble(j)/(2.d0*pi*dble(i)))/pi)
   fla_org(lvl) = asin(tan(pi-thetaY)*dble(j(lvl))/(2.d0*pi*dble(i(lvl))))/pi

elseif(condLHS.ge.condRHS)then ! full wetting
 
   case = 2
  
   nu = (2.d0*pi*dble(i(lvl))/dble(j(lvl)))
   par1 = (nu/sqrt(1.d0+(nu**2)))   
   call comelp(par1,junk,ee3)

   fsl(lvl) = abs((2.d0*sqrt(1.d0+(nu**2))/pi)*ee3)
   fsl_org(lvl) = abs((2.d0*sqrt(1.d0+(nu**2))/pi)*ee3)
   fla(lvl) = 0.d0
   fla_org = 0.d0

endif
! -----------------------------------------------------------------------------------------


return
end
!######################################################################################################################################
!######################################################################################################################################



!==========================================================================================================
!==========================================================================================================
Subroutine pillar3D()

use variables

implicit none

real*8 fla_num, fla_num_org, dstar, angleAlpha, beta, Wsq, Dsq

! fsl and fla have been calculated for the 2D case by the call on pillar() in main.F90 before the call on pillar3D
! From these values, we need to know the numerator of fla
! For that we need again to distinquish cases (Touching - No_Touching)


if(case.eq.1)then    ! NO TOUCHING OF THE SURFACE

   beta = asin((dble(j(lvl))-dble(i(lvl)))/(2.d0*RsagStar(lvl)))
   fla_num = RsagStar(lvl)*2.d0*beta

elseif(case.eq.3)then

   ! calculate the dimensionless distance d
   dstar = (dble(j(lvl)) - dble(i(lvl)))/2.d0
   angleAlpha = acos((HeightStar(lvl) + sqrt(RsagStar(lvl)**2 - dstar**2))/RsagStar(lvl))
   beta = asin((dble(j(lvl))-dble(i(lvl)))/(2.d0*RsagStar(lvl)))
   fla_num = 2.d0*RsagStar(lvl)*(beta-angleAlpha)

endif

fla_num_org = dble(j(lvl)) - dble(i(lvl))

! these are the 3D values
fsl(lvl) = 0.d0  
fla(lvl) = 0.d0  
fsl_org(lvl) = 0.d0
fla_org(lvl) = 0.d0

! Calculate variables of the 3D pillars that go into the calculation of fsl and fla
Wsq = dble(i(lvl))*dble(i(lvl))
Dsq = dble(j(lvl))*dble(j(lvl))

! Now store in the initial variables fsl(lvl) and fla(lvl) the 3D calculated values
fsl(lvl) = Wsq/Dsq
fla(lvl) = ( 2.d0*fla_num*dble(i(lvl)) + (fla_num*fla_num) ) / Dsq

fsl_org(lvl) = Wsq/Dsq
fla_org(lvl) = ( 2.d0*fla_num_org*dble(i(lvl)) + (fla_num_org*fla_num_org) ) / Dsq


return
end
!==========================================================================================================
!==========================================================================================================



